=== modified file 'drizzled/definition/cache.cc'
--- drizzled/definition/cache.cc	2010-11-18 22:57:33 +0000
+++ drizzled/definition/cache.cc	2010-11-19 03:03:40 +0000
@@ -20,7 +20,9 @@
 
 #include "config.h"
 
-#include "drizzled/pthread_globals.h"
+#include <boost/thread/mutex.hpp>
+#include <boost/thread/shared_mutex.hpp>
+
 #include "drizzled/session.h"
 #include "drizzled/identifier/table.h"
 #include "drizzled/definition/cache.h"
@@ -32,9 +34,9 @@
 
 TableShare::shared_ptr Cache::find(const TableIdentifier &identifier)
 {
-  //safe_mutex_assert_owner(LOCK_open.native_handle);
+  boost::mutex::scoped_lock scopedLock(_mutex);
 
-  CacheMap::iterator iter= cache.find(identifier.getKey());
+  Map::iterator iter= cache.find(identifier.getKey());
   if (iter != cache.end())
   {
     return (*iter).second;
@@ -45,14 +47,15 @@
 
 void Cache::erase(const TableIdentifier &identifier)
 {
-  //safe_mutex_assert_owner(LOCK_open.native_handle);
+  boost::mutex::scoped_lock scopedLock(_mutex);
   
   cache.erase(identifier.getKey());
 }
 
 bool Cache::insert(const TableIdentifier &identifier, TableShare::shared_ptr share)
 {
-  std::pair<CacheMap::iterator, bool> ret=
+  boost::mutex::scoped_lock scopedLock(_mutex);
+  std::pair<Map::iterator, bool> ret=
     cache.insert(std::make_pair(identifier.getKey(), share));
 
   return ret.second;

=== modified file 'drizzled/definition/cache.h'
--- drizzled/definition/cache.h	2010-11-18 22:57:33 +0000
+++ drizzled/definition/cache.h	2010-11-19 03:03:40 +0000
@@ -25,15 +25,18 @@
 
 namespace drizzled {
 
+namespace generator {
+class TableDefinitionCache;
+}
+
 namespace definition {
 
-typedef boost::unordered_map< TableIdentifier::Key, TableShare::shared_ptr> CacheMap;
-
 class Cache
 {
-  CacheMap cache;
-
 public:
+
+typedef boost::unordered_map< TableIdentifier::Key, TableShare::shared_ptr> Map;
+
   static inline Cache &singleton()
   {
     static Cache open_cache;
@@ -41,11 +44,6 @@
     return open_cache;
   }
 
-  CacheMap &getCache()
-  {
-    return cache;
-  }
-
   size_t size() const
   {
     return cache.size();
@@ -56,9 +54,26 @@
     cache.rehash(arg);
   }
 
+  boost::mutex &mutex()
+  {
+    return _mutex;
+  }
+
   TableShare::shared_ptr find(const TableIdentifier &identifier);
   void erase(const TableIdentifier &identifier);
   bool insert(const TableIdentifier &identifier, TableShare::shared_ptr share);
+
+protected:
+  friend class drizzled::generator::TableDefinitionCache;
+
+  Map &getCache()
+  {
+    return cache;
+  }
+
+private:
+  Map cache;
+  boost::mutex _mutex;
 };
 
 } /* namespace definition */

=== modified file 'drizzled/definition/table.cc'
--- drizzled/definition/table.cc	2010-11-19 00:00:46 +0000
+++ drizzled/definition/table.cc	2010-11-19 03:03:40 +0000
@@ -973,8 +973,8 @@
   uint32_t local_null_fields= 0;
   reclength= 0;
 
-  vector<uint32_t> field_offsets;
-  vector<uint32_t> field_pack_length;
+  std::vector<uint32_t> field_offsets;
+  std::vector<uint32_t> field_pack_length;
 
   field_offsets.resize(fields);
   field_pack_length.resize(fields);

=== modified file 'drizzled/definition/table.h'
--- drizzled/definition/table.h	2010-11-18 22:57:33 +0000
+++ drizzled/definition/table.h	2010-11-19 03:03:40 +0000
@@ -61,8 +61,10 @@
 class TableShare
 {
   typedef std::vector<std::string> StringVector;
+
 public:
   typedef boost::shared_ptr<TableShare> shared_ptr;
+  typedef std::vector <shared_ptr> vector;
 
   TableShare(TableIdentifier::Type type_arg);
 

=== modified file 'drizzled/generator.h'
--- drizzled/generator.h	2010-10-22 07:38:49 +0000
+++ drizzled/generator.h	2010-11-19 03:03:40 +0000
@@ -23,6 +23,7 @@
 
 #include "drizzled/session.h"
 
+#include "drizzled/generator/table_definition_cache.h"
 #include "drizzled/generator/event_observers.h"
 #include "drizzled/generator/functions.h"
 #include "drizzled/generator/schema.h"

=== added file 'drizzled/generator/table_definition_cache.h'
--- drizzled/generator/table_definition_cache.h	1970-01-01 00:00:00 +0000
+++ drizzled/generator/table_definition_cache.h	2010-11-19 03:03:40 +0000
@@ -0,0 +1,73 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_GENERATOR_TABLE_DEFINITION_CACHE_H
+#define DRIZZLED_GENERATOR_TABLE_DEFINITION_CACHE_H
+
+#include <boost/bind.hpp>
+#include <boost/thread/mutex.hpp>
+#include "drizzled/definition/cache.h"
+
+namespace drizzled {
+namespace generator {
+
+class TableDefinitionCache
+{
+  drizzled::TableShare::vector local_vector;
+  drizzled::TableShare::vector::iterator iter;
+
+public:
+
+  TableDefinitionCache()
+  {
+    boost::mutex::scoped_lock scopedLock(definition::Cache::singleton().mutex());
+
+    local_vector.reserve(definition::Cache::singleton().size());
+
+    std::transform(definition::Cache::singleton().getCache().begin(),
+                   definition::Cache::singleton().getCache().end(),
+                   std::back_inserter(local_vector),
+                   boost::bind(&definition::Cache::Map::value_type::second, _1) );
+    assert(local_vector.size() == definition::Cache::singleton().getCache().size());
+
+    iter= local_vector.begin();
+  }
+
+  ~TableDefinitionCache()
+  {
+  }
+
+  operator drizzled::TableShare::shared_ptr()
+  {
+    while (iter != local_vector.end())
+    {
+      drizzled::TableShare::shared_ptr ret(*iter);
+      iter++;
+      return ret;
+    }
+
+    return drizzled::TableShare::shared_ptr();
+  }
+};
+
+} /* namespace generator */
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_GENERATOR_TABLE_DEFINITION_CACHE_H */

=== modified file 'drizzled/include.am'
--- drizzled/include.am	2010-11-13 17:32:40 +0000
+++ drizzled/include.am	2010-11-19 03:03:40 +0000
@@ -609,6 +609,7 @@
 				 drizzled/generator/functions.cc \
 				 drizzled/generator/schema.cc \
 				 drizzled/generator/table.cc \
+				 drizzled/generator/table_definition_cache.h \
 				 drizzled/ha_commands.cc \
 				 drizzled/hybrid_type_traits.cc \
 				 drizzled/hybrid_type_traits_decimal.cc \

=== modified file 'plugin/table_cache_dictionary/table_definition_cache.cc'
--- plugin/table_cache_dictionary/table_definition_cache.cc	2010-11-08 21:03:52 +0000
+++ plugin/table_cache_dictionary/table_definition_cache.cc	2010-11-19 03:03:40 +0000
@@ -21,7 +21,6 @@
 #include "config.h"
 
 #include "plugin/table_cache_dictionary/dictionary.h"
-#include "drizzled/pthread_globals.h"
 
 using namespace drizzled;
 using namespace std;
@@ -37,80 +36,43 @@
 }
 
 table_cache_dictionary::TableDefinitionCache::Generator::Generator(drizzled::Field **arg) :
-  drizzled::plugin::TableFunction::Generator(arg),
-  is_primed(false)
+  drizzled::plugin::TableFunction::Generator(arg)
 {
-  LOCK_open.lock(); /* Optionally lock for remove tables from open_cahe if not in use */
 }
 
 table_cache_dictionary::TableDefinitionCache::Generator::~Generator()
 {
-  LOCK_open.unlock(); /* Optionally lock for remove tables from open_cahe if not in use */
-}
-
-bool table_cache_dictionary::TableDefinitionCache::Generator::nextCore()
-{
-  if (is_primed)
-  {
-    table_share_iterator++;
-  }
-  else
-  {
-    is_primed= true;
-    table_share_iterator= definition::Cache::singleton().getCache().begin();
-  }
-
-  if (table_share_iterator == definition::Cache::singleton().getCache().end())
-    return false;
-
-  share= (*table_share_iterator).second;
-
-  return true;
-}
-
-bool table_cache_dictionary::TableDefinitionCache::Generator::next()
-{
-  while (not nextCore())
-  {
-    if (table_share_iterator != definition::Cache::singleton().getCache().end())
-      continue;
-
-    return false;
-  }
-
-  return true;
 }
 
 bool table_cache_dictionary::TableDefinitionCache::Generator::populate()
 {
-  if (not next())
-    return false;
-  
-  fill();
-
-  return true;
-}
-
-void table_cache_dictionary::TableDefinitionCache::Generator::fill()
-{
-  /**
-    For test cases use:
-    --replace_column 3 # 4 # 5 #
-  */
-
-  /* TABLE_SCHEMA 1 */
-  string arg;
-  push(share->getSchemaName(arg));
-
-  /* TABLE_NAME  2 */
-  push(share->getTableName(arg));
-
-  /* VERSION 3 */
-  push(static_cast<int64_t>(share->getVersion()));
-
-  /* TABLE_COUNT 4 */
-  push(static_cast<uint64_t>(share->getTableCount()));
-
-  /* IS_NAME_LOCKED */
-  push(false);
+  drizzled::TableShare::shared_ptr share;
+
+  while ((share= table_definition_cache_generator))
+  {
+    /**
+      For test cases use:
+      --replace_column 3 # 4 # 5 #
+    */
+
+    /* TABLE_SCHEMA 1 */
+    string arg;
+    push(share->getSchemaName(arg));
+
+    /* TABLE_NAME  2 */
+    push(share->getTableName(arg));
+
+    /* VERSION 3 */
+    push(static_cast<int64_t>(share->getVersion()));
+
+    /* TABLE_COUNT 4 */
+    push(static_cast<uint64_t>(share->getTableCount()));
+
+    /* IS_NAME_LOCKED */
+    push(false);
+
+    return true;
+  }
+
+  return false;
 }

=== modified file 'plugin/table_cache_dictionary/table_definition_cache.h'
--- plugin/table_cache_dictionary/table_definition_cache.h	2010-11-18 22:57:33 +0000
+++ plugin/table_cache_dictionary/table_definition_cache.h	2010-11-19 03:03:40 +0000
@@ -20,8 +20,7 @@
 #ifndef PLUGIN_TABLE_CACHE_DICTIONARY_TABLE_DEFINITION_CACHE_H
 #define PLUGIN_TABLE_CACHE_DICTIONARY_TABLE_DEFINITION_CACHE_H
 
-#include "drizzled/definition/cache.h"
-#include "drizzled/definition/table.h"
+#include "drizzled/generator.h"
 
 namespace table_cache_dictionary {
 
@@ -32,14 +31,7 @@
 
   class Generator : public drizzled::plugin::TableFunction::Generator 
   {
-    bool is_primed;
-    drizzled::TableShare::shared_ptr share;
-    drizzled::definition::CacheMap::const_iterator table_share_iterator;
-
-    void fill();
-
-    bool nextCore();
-    bool next();
+    drizzled::generator::TableDefinitionCache table_definition_cache_generator;
 
   public:
     bool populate();

